@@ -485,6 +485,25 @@ class GlobalSettings:
             [26523],
             [41471],
             [2936],
+            [85, 85],
+        ],
+        "nerdstash_v2": [
+            [3],
+            [49356],
+            [1431],
+            [31715],
+            [34387],
+            [20765],
+            [30702],
+            [10691],
+            [49333],
+            [1266],
+            [19438],
+            [43145],
+            [26523],
+            [41471],
+            [2936],
+            [85, 85],
         ],
     }
 
@@ -607,7 +626,177 @@ class GlobalSettings:
         [34650],
     ]
 
-    _DINKUS_ASTERISM = BiasGroup(-0.12).add("***", "⁂")
+    # whitelist
+    _REP_PEN_WHITELIST = {
+        "gpt2": [],
+        "gpt2-genji": [],
+        "pile": [],
+        "nerdstash_v1": [
+            "'",
+            '"',
+            ",",
+            ".",
+            ":",
+            "\n",
+            "ve",
+            "s",
+            "t",
+            "n",
+            "d",
+            "ll",
+            "re",
+            "m",
+            "-",
+            "*",
+            ")",
+            " the",
+            " a",
+            " an",
+            " and",
+            " or",
+            " not",
+            " no",
+            " is",
+            " was",
+            " were",
+            " did",
+            " does",
+            " isn",
+            " wasn",
+            " weren",
+            " didn",
+            " doesn",
+            " him",
+            " her",
+            " his",
+            " hers",
+            " their",
+            " its",
+            " could",
+            " couldn",
+            " should",
+            " shouldn",
+            " would",
+            " wouldn",
+            " have",
+            " haven",
+            " had",
+            " hadn",
+            " has",
+            " hasn",
+            " can",
+            " cannot",
+            " are",
+            " aren",
+            " will",
+            " won",
+            "0",
+            "1",
+            "2",
+            "3",
+            "4",
+            "5",
+            "6",
+            "7",
+            "8",
+            "9",
+            '."',
+            ',"',
+            "====",
+            " ",
+        ],
+        "nerdstash_v2": [
+            "'",
+            '"',
+            ",",
+            ".",
+            ":",
+            "\n",
+            "ve",
+            "s",
+            "t",
+            "n",
+            "d",
+            "ll",
+            "re",
+            "m",
+            "-",
+            "*",
+            ")",
+            " the",
+            " a",
+            " an",
+            " and",
+            " or",
+            " not",
+            " no",
+            " is",
+            " was",
+            " were",
+            " did",
+            " does",
+            " isn",
+            " wasn",
+            " weren",
+            " didn",
+            " doesn",
+            " him",
+            " her",
+            " his",
+            " hers",
+            " their",
+            " its",
+            " could",
+            " couldn",
+            " should",
+            " shouldn",
+            " would",
+            " wouldn",
+            " have",
+            " haven",
+            " had",
+            " hadn",
+            " has",
+            " hasn",
+            " can",
+            " cannot",
+            " are",
+            " aren",
+            " will",
+            " won",
+            "0",
+            "1",
+            "2",
+            "3",
+            "4",
+            "5",
+            "6",
+            "7",
+            "8",
+            "9",
+            '."',
+            ',"',
+            "====",
+            " ",
+            "'t've",
+            "'s",
+            "'t",
+            "'ve",
+            "'n",
+            "'ll",
+            "'d",
+            "'re",
+            "'m",
+        ],
+    }
+
+    _DINKUS_ASTERISM = {
+        "gpt2": BiasGroup(-0.12).add("***", "⁂"),
+        "gpt2-genji": None,
+        "pile": BiasGroup(-0.12).add("***"),
+        "nerdstash_v1": BiasGroup(-0.08).add("***", "⁂"),
+        "nerdstash_v2": BiasGroup(-0.08).add("***", "⁂"),
+    }
 
     _DEFAULT_SETTINGS = {
         "generate_until_sentence": False,
@@ -615,6 +804,7 @@ class GlobalSettings:
         "ban_brackets": True,
         "bias_dinkus_asterism": False,
         "ban_ambiguous_genji_tokens": True,
+        "rep_pen_whitelist": True,
     }
 
     # type completion for __setitem__ and __getitem__
@@ -629,6 +819,8 @@ class GlobalSettings:
         bias_dinkus_asterism: bool
         #: Apply the GENJI_AMBIGUOUS_TOKENS if model is Genji
         ban_ambiguous_genji_tokens: bool
+        #: Apply the REP_PEN_WHITELIST (repetition penalty whitelist)
+        rep_pen_whitelist: bool
 
     #: Value to set num_logprobs at to disable logprobs
     NO_LOGPROBS = -1
@@ -689,6 +881,7 @@ class GlobalSettings:
             "num_logprobs": self._settings["num_logprobs"],
             "bad_words_ids": [],
             "logit_bias_exp": [],
+            "repetition_penalty_whitelist": [],
             "return_full_text": False,
             "use_string": False,
             "use_cache": False,
@@ -707,6 +900,17 @@ class GlobalSettings:
             settings["bad_words_ids"].extend(self._GENJI_AMBIGUOUS_TOKENS)
 
         if self._settings["bias_dinkus_asterism"]:
-            settings["logit_bias_exp"].extend(self._DINKUS_ASTERISM.get_tokenized_entries(model))
+            assert (
+                tokenizer_name in self._DINKUS_ASTERISM
+            ), f"Tokenizer {tokenizer_name} not supported with bias_dinkus_asterism"
+
+            bias = self._DINKUS_ASTERISM[tokenizer_name]
+            if bias is not None:
+                settings["logit_bias_exp"].extend(bias.get_tokenized_entries(model))
+
+        if self._settings["rep_pen_whitelist"]:
+            settings["repetition_penalty_whitelist"].extend(
+                Tokenizer.encode(model, tok) for tok in self._REP_PEN_WHITELIST[tokenizer_name]
+            )
 
         return settings
